import typing

from pydantic import BaseModel, model_validator

from .exceptions import CommunicationSystemException


class CircuitInstruction(BaseModel):
    type: str | None
    register_indexes: typing.List[int] = []

    @model_validator(mode="before")
    @classmethod
    def _check_instruction(cls, values: typing.Dict) -> typing.Dict:
        if not values["register_indexes"]:
            raise CommunicationSystemException(
                "Instruction doesn't contain any register index"
            )
        match values["type"]:
            case "cnot":
                if len(values["register_indexes"]) > 2:
                    raise CommunicationSystemException("Wrong register indexes number")
            case "hadamard":
                if len(values["register_indexes"]) > 1:
                    raise CommunicationSystemException("Wrong register indexes number")
            case _:
                raise CommunicationSystemException("Wrong gate type")
        return values
