import signal
import socketserver
import typing

from communication_system import CommunicationSystem
from communication_system.exceptions import CommunicationSystemException
from communication_system.motherboard import Motherboard
from secret import flag


def start_communication_system() -> CommunicationSystem:
    motherboard: Motherboard = Motherboard()
    communication_system: CommunicationSystem = CommunicationSystem(
        motherboard=motherboard
    )
    return communication_system


def connect_to_system(req: typing.Any) -> None:
    try:
        req.sendall(
            b"\n|--------------------------------------------|\n"
            + b"| Quantum Renegades Communication System     |\n"
            + b"|--------------------------------------------|\n"
            + b"| + System initilization  [Success!]         |\n"
            + b"| + Quantum Circuit tests [Failed...]        |\n"
            + b"| + Receiver decoding     [Success!]         |\n"
            + b"|                                            |\n"
            + b"| > Input instruction set for fix...         |\n"
            + b"|                                            |\n"
            + b"|--------------------------------------------|\n"
            + b"\n> "
        )
        input: typing.List = req.recv(4096).decode().strip().split(";")
        req.sendall(b"\n % Testing quantum circuit, please wait...\n")
        tests_passes = []
        for _ in range(100):
            communication_system: CommunicationSystem = start_communication_system()
            communication_system.add_instructions(input)
            communication_system.measure_qubits()
            communication_system.decode()
            tests_passes.append(communication_system.test_output())
        if all(tests_passes) and tests_passes:
            req.sendall(f"\n{flag}\n".encode())
        else:
            req.sendall(b"Testing suite failed...\n")
        req.close()
        exit()
    except CommunicationSystemException as cse:
        try:
            req.sendall(f"\n{cse}\n".encode())
        except Exception:
            pass
    except Exception as e:
        try:
            req.sendall(b"\nUnexpected error occured...\n")
        except Exception:
            pass
    req.close()
    exit()


def main():
    class IncomingConnection(socketserver.BaseRequestHandler):
        def handle(self):
            signal.alarm(300)
            req: typing.Any = self.request
            print("starting server...")
            while True:
                connect_to_system(req=req)

    class ReusableTCPServer(socketserver.ForkingMixIn, socketserver.TCPServer):
        pass

    socketserver.TCPServer.allow_reuse_address = False
    server: ReusableTCPServer = ReusableTCPServer(
        ("0.0.0.0", 1337), IncomingConnection
    )
    server.serve_forever()


if __name__ == "__main__":
    main()
